# $Id: controlwin.py,v 1.5 2002/02/18 13:01:37 marijn Exp $

# An air traffic simulation game. 
# Copyright (C) 2000,1,2  Marijn Vriens <marijn@sanity.dhs.org>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

import math
import GDK
import GTK
import gnome.affine

import simclient
import abswindow

class ControlWin(abswindow.AbsWindow):
    WindowNickName = "control"
    def _build(self):
        ui = abswindow.WindowList().getUserInterface()
        self._currFlightNr = ui.get_widget("controlFlightCurr")
        self._currAltitude = ui.get_widget("controlAltitudeCurr")
        self._currSpeed    = ui.get_widget("controlSpeedCurr")
        self._currHeading  = ui.get_widget("controlHeadingCurr")
        self._orderAltitude= ui.get_widget("controlAltitudeOrder")
        self._orderHeading = ui.get_widget("controlHeadingOrder")
        self._orderSpeed   = ui.get_widget("controlSpeedOrder")

        self._dispHeading  = HeadingDisplay(ui.get_widget("headingDisplay"),
                                            self._changeHeadingDial)

        self._speedAdj = ui.get_widget("controlSpeedSlider").get_adjustment()
        self._speedAdj.connect("value-changed", self._changeSpeedSlider)
        self._altitudeAdj = ui.get_widget("controlAltSlider").get_adjustment()
        self._altitudeAdj.connect("value-changed", self._changeAltitudeSlider)
        
        ui.get_widget("controlSendBtn").connect("clicked", self._clickOrderButton)
    def setEntityToDisplay(self, entity):
        self._entity = entity
        self._setText()
        self._setSliders()
        self._setHeading()
    def updateDisplay(self):
        self._setCurrText()
        self._updateHeading()
    def _setSliders(self):
        self._speedAdj.set_all(self._entity.speed(),
                               self._entity.getType().minSpeed(),
                               self._entity.getType().maxSpeed(),
                               1, 5, 0)
        self._altitudeAdj.set_all(self._entity.height(),
                                  self._entity.getType().minAltitude(),
                                  self._entity.getType().maxAltitude(),
                                  1, 5, 0)
    def _setText(self):
        self._setCurrText()
        self._setOrderText()
    def _setCurrText(self):
        self._currFlightNr.set_text(self._entity.flightNumber())
        self._currAltitude.set_text(str(int(self._entity.height())))
        self._currSpeed.set_text(str(int(self._entity.speed())))
        self._currHeading.set_text(str(int(self._entity.heading())))        
    def _setOrderText(self):
        self._orderAltitude.set_text(str(int(self._entity.height())))
        self._orderSpeed.set_text(str(int(self._entity.speed())))
        self._orderHeading.set_text(str(int(self._entity.heading())))
    def _updateHeading(self):
        self._dispHeading.updateCurrHeading(self._entity.heading())
    def _setHeading(self):
        self._dispHeading.updateAllHeading(self._entity.heading())
    def _clickOrderButton(self, widget):
        self._sendOrder()
    def _changeSpeedSlider(self, widget):
        self._orderSpeed.set_text(str(int(self._speedAdj.value)))
    def _changeAltitudeSlider(self, widget):
        self._orderAltitude.set_text(str(int(self._altitudeAdj.value)))
    def _changeHeadingDial(self, degrees):
        self._orderHeading.set_text(str(int(degrees)))
    def _sendOrder(self):
        order = {}
        order['speed']   = self._speedAdj.value
        order['altitude']= self._altitudeAdj.value
        order['heading'] = int(self._orderHeading.get())
        simclient.SimClient().giveOrder(None, self._entity.id(), order)

class HeadingDisplay:
    def __init__(self, widget, headingChangeFunc):
        self._window = widget
        self._headingChangeFunc = headingChangeFunc
        self._root   = widget.root()
        self._setupCanvas()
        
        self._drawBackground(self._root)
        self._controls = None
    def updateAllHeading(self, heading):
        if(self._controls == None):
            self._headingGroup = self._drawCurrentHeading(self._root)
            self._orderGroup   = self._drawOrderedHeading(self._root)
            self._trackerGroup = self._drawPointerTracker(self._root)
            self._window.connect("event", self._eventCallback)
            self._controls = 1
        self._updatePointerGroup(self._headingGroup, _degrees2pi(heading))
        self._updatePointerGroup(self._orderGroup, _degrees2pi(heading))
    def updateCurrHeading(self, heading):
        self._updatePointerGroup(self._headingGroup, _degrees2pi(heading))
    def _setupCanvas(self):
        # setup so that 0,0 is the center of the canvas, with radius width/2
        (width, height) = self._window.size_request()
        self._width = width /2.0
        self._height= height/2.0
        self._window.set_scroll_region(-self._width, -self._height,
                                       self._width , self._height)
        self._window.set_pixels_per_unit(1)
    def _drawBackground(self, root):
        group = root.add("group")
        markWidth = self._width /100.0 * 90
        markHeight= self._height/100.0 * 90
    
        #print markWidth, markHeight
        for i in range(0, 360, 10):
            if(i % 90) == 0:
                innerRadio = markWidth/100.0 * 70
                outerRadio = markWidth/100.0 * 110
                lineWidth  = 2
            elif(i % 30) == 0:
                innerRadio = markWidth
                outerRadio = markWidth/100.0 * 110
                lineWidth  = 1
            else:
                innerRadio = markWidth
                outerRadio = markWidth+2
                lineWidth  = 0.5
            x = math.sin(_degrees2pi(i))
            y = math.cos(_degrees2pi(i))
            group.add("line", fill_color="black", width_units=lineWidth,
                      points=(x*innerRadio, y*innerRadio,
                              x*outerRadio, y*outerRadio))
    def _drawCurrentHeading(self, root):
        group = root.add("group")
        frontPoint = -self._width
        backPoint  = self._width / 100.0 * 80
        sideArrow  = self._width / 100.0 * 20
        pointArrowFront = frontPoint + (self._width / 100.0 * 40)
        pointArrowBack  = backPoint - (self._width / 100.0 * 40)
        group.add("line", fill_color="red", width_units=2,
                  points=(0, frontPoint,
                          0, backPoint))
        group.add("polygon", fill_color="red", width_units=2,
                  points=(0, frontPoint,
                          sideArrow, pointArrowFront,
                          -sideArrow, pointArrowFront,
                          0, frontPoint))
        #group.add("polygon", fill_color="red", width_units=2,
        #          points=(0, pointArrowBack,
        #                  sideArrow, backPoint,
        #                  -sideArrow, backPoint,
        #                  0, pointArrowBack))
        return group
    def _drawOrderedHeading(self, root):
        group = root.add("group")
        outerPoint = -self._width
        sideTail  = -self._width / 100.0 * 10
        innerTail  = -self._width / 100.0 * 20
        widthTail  = self._width / 100.0 * 20
        group.add("polygon", fill_color="blue", width_units=1,
                  points=(0, outerPoint,
                          widthTail, sideTail,
                          0, innerTail,
                          -widthTail, sideTail,
                          0, outerPoint))
        return group
    def _drawPointerTracker(self, root):
        group = root.add("group")
        outerRadio = -self._width
        innerRadio = -self._width / 100.0 * 60
        ellipseSize= self._width / 100.0 * 30
        group.add("line", fill_color="blue", width_units=1,
                  points=(0, outerRadio,
                          0, innerRadio))
        textGroup = root.add("group")
        textGroup.add("ellipse", fill_color="grey",
                      x1=ellipseSize, y1=ellipseSize,
                      x2=-ellipseSize, y2=-ellipseSize)
        self._degreeDisplay = textGroup.add("text", text="NONE", font='fixed',
                                            anchor=GTK.ANCHOR_CENTER,
                                            fill_color='black', x=0, y=0)
        return group
    def _updatePointerGroup(self, group, rad):
        group.affine_absolute(gnome.affine.rotate(rad))
    def _eventCallback(self, widget, event):
        if(event.type == GDK.MOTION_NOTIFY or
           event.type == GDK.ENTER_NOTIFY):
            rad = self._getLocation(event)
            self._updatePointerGroup(self._trackerGroup ,rad)
            self._degreeDisplay.set(text=str(int(_rad2degrees(rad))))
        elif(event.type == GDK.BUTTON_PRESS):
            rad = self._getLocation(event)
            self._updatePointerGroup(self._orderGroup, rad)
            self._headingChangeFunc(_rad2degrees(rad))
    def _getLocation(self, event):
        x = event.x-self._width
        y = self._height-event.y
        rad = -(math.atan2(y,x) - math.pi/2)            
        if(rad < 0):
            rad = 2*math.pi + rad
        return rad

def _rad2degrees(rad):
    return rad * ( 180 / math.pi )
def _degrees2pi(deg):
    return deg * ( math.pi / 180 )

# $Log: controlwin.py,v $
# Revision 1.5  2002/02/18 13:01:37  marijn
# Silenced some debug info
#
# Revision 1.4  2002/02/10 18:09:53  marijn
# Improved order information feedback
#
# Revision 1.3  2002/02/10 02:45:22  marijn
# Added heading control
#
# Revision 1.2  2002/02/07 16:25:21  marijn
# small fixes
#
# Revision 1.1  2002/02/07 12:54:06  marijn
# Reordered windows, Added plane ordering functionality
#
